package com.yizhi.site.application.service.impl;

import java.util.*;
import java.util.logging.Logger;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import com.yizhi.site.application.domain.DataClassification;
import com.yizhi.site.application.mapper.DataClassificationMapper;
import com.yizhi.site.application.mapper.InformationMapper;
import com.yizhi.site.application.service.DataClassificationService;
import com.yizhi.site.application.constant.SiteConstant;
import com.yizhi.site.application.service.InformationService;
import com.yizhi.site.application.service.PortalBannerService;
import com.yizhi.site.application.vo.domain.DataClassificationVo;
import com.yizhi.site.application.vo.site.IdTwoVO;
import com.yizhi.site.application.vo.site.RespVO;
import com.yizhi.site.application.vo.site.SiteComponyIdVO;
import com.yizhi.site.application.vo.site.ClassificationVO;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.orm.id.IdGenerator;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author bob123
 * @since 2018-03-27
 */
@Service
@Slf4j
public class DataClassificationServiceImpl extends ServiceImpl<DataClassificationMapper, DataClassification>
		implements DataClassificationService {

	@Autowired
	private DataClassificationMapper dataClassificationMapper;
	@Autowired
	IdGenerator idGenerator;
	@Autowired
	DataClassificationService dataClassificationService;
	@Autowired
    PortalBannerService portalBannerService;

	@Autowired
    private InformationService informationService;

	@Autowired
    private InformationMapper informationMapper;

    @Override
	public List<DataClassification> getSon(Long parentId) {
		DataClassification data = new DataClassification();
		data.setParentId(parentId);
		EntityWrapper<DataClassification> wrapper = new EntityWrapper<DataClassification>(data);
		return this.selectList(wrapper);
	}

	@Override
	public List<ClassificationVO> getClassification(SiteComponyIdVO siteComponyIdVO) {
		Long siteId = siteComponyIdVO.getSiteId();
		log.info("查询的站点id:{}",siteId);
		Long createById = siteComponyIdVO.getCreateById();
		String createByName = siteComponyIdVO.getCreateByName();
		ClassificationVO classificationVO = null;
		Date date = new Date();
		// 防止并发添加数据
		List<ClassificationVO> list = dataClassificationMapper.selectSonClassification(siteId);
		if (list == null || list.size() < 1) {
			/*//没有初始化的数据，2024-12-16注释
			synchronized (this) {
				DataClassification dataClassification = new DataClassification();

                dataClassification.setId(idGenerator.generate());
                dataClassification.setName("公告");
                dataClassification.setDescription("公告");
                dataClassification.setParentId(0L);
                dataClassification.setSiteId(siteId);
                dataClassification.setCreateById(createById);
                dataClassification.setCreateByName(createByName);
                dataClassification.setUpdateById(createById);
                dataClassification.setUpdateByName(createByName);
                dataClassification.setSort(1L);
                dataClassification.setState(1);
                dataClassification.setCreateTime(date);
                dataClassification.setUpdateTime(date);
                dataClassificationService.insert(dataClassification);
                classificationVO = new ClassificationVO();
                BeanUtils.copyProperties(dataClassification, classificationVO);
                list.add(classificationVO);


				dataClassification.setId(idGenerator.generate());
				dataClassification.setName("新闻");
				dataClassification.setDescription("新闻");
				dataClassification.setParentId(0L);
				dataClassification.setSiteId(siteId);
				dataClassification.setCreateById(createById);
				dataClassification.setCreateByName(createByName);
				dataClassification.setUpdateById(createById);
				dataClassification.setUpdateByName(createByName);
				dataClassification.setSort(2L);
				dataClassification.setState(1);
				dataClassification.setCreateTime(date);
				dataClassification.setUpdateTime(date);
				dataClassificationService.insert(dataClassification);
				classificationVO = new ClassificationVO();
				BeanUtils.copyProperties(dataClassification, classificationVO);
				list.add(classificationVO);

			}*/
		} else {
			Map<String, Object> map = null;
			for(ClassificationVO vo : list) {
				if (vo.getId() != 0L) {
					EntityWrapper<DataClassification> wrapper = new EntityWrapper<>();
					wrapper.eq("state", 1);
					wrapper.eq("parent_id", vo.getId());
					List<DataClassification> selectByMap = dataClassificationService.selectList(wrapper);
					List<DataClassificationVo> selectVoByMap = new ArrayList<>();
					if (CollectionUtil.isNotEmpty(selectByMap)) {
						for (DataClassification d : selectByMap) {
							DataClassificationVo entityVo = new DataClassificationVo();
							BeanUtils.copyProperties(d, entityVo);
							selectVoByMap.add(entityVo);

							//获取子节点
							EntityWrapper<DataClassification> wrapperChild = new EntityWrapper<>();
							wrapperChild.eq("state", 1);
							wrapperChild.eq("parent_id", entityVo.getId());
							List<DataClassification> classificationChild = dataClassificationService.selectList(wrapperChild);
							List<DataClassificationVo> childClassifyNews = new ArrayList<>();
							if (CollectionUtil.isNotEmpty(classificationChild)) {
								for (DataClassification child : classificationChild) {
									DataClassificationVo entityChild = new DataClassificationVo();
									BeanUtils.copyProperties(child, entityChild);
									childClassifyNews.add(entityChild);
								}
								entityVo.setChildClassifyNews(childClassifyNews);
								;
							}
						}
					}
					vo.setData(selectVoByMap);
				}
			}
		}
		return list;
	}

	@Override
	public Boolean insertClassification(DataClassification dataClassification) {
		Long sort = dataClassificationMapper.selectMaxSort(dataClassification.getParentId());
		if (sort == null) {
			sort = 0L;
		}
		dataClassification.setSort(sort + 1);
		dataClassification.setId(idGenerator.generate());
		dataClassification.setState(SiteConstant.FIRST);
		return this.insert(dataClassification);
	}

	@Override
	public List<DataClassification> selectParent(RequestContext context) {
		DataClassification dataClassification = new DataClassification();
		dataClassification.setSiteId(context.getSiteId());
		dataClassification.setParentId(0L);
		EntityWrapper<DataClassification> wrapper = new EntityWrapper<DataClassification>(dataClassification);
		return this.selectList(wrapper);
	}

	@Override
	public Boolean dragClassify(IdTwoVO vo) {
		Date date = new Date();
		Long sourceId = vo.getSourceId();
		Long targetId = vo.getTargetId();
		DataClassification source = dataClassificationService.selectById(sourceId);
		DataClassification target = dataClassificationService.selectById(targetId);
		Long sourceSort = source.getSort();
		Long targetSort = target.getSort();
		source.setSort(targetSort);
		source.setUpdateById(vo.getAccountId());
		source.setUpdateByName(vo.getAccountName());
		source.setUpdateTime(date);
		dataClassificationService.updateById(source);
		target.setSort(sourceSort);
		target.setUpdateById(vo.getAccountId());
		target.setUpdateByName(vo.getAccountName());
		target.setUpdateTime(date);
		dataClassificationService.updateById(target);
		return true;
	}

    /**
     *  删除儿子分类
     * @param id 二级分类的Id
     * @return
     */
    @Override
    @Transactional
    public RespVO delClassify(Long id) {

      Integer count =   informationMapper.queryInformationByTypeTwo(id);
        if (count>0){
            RespVO respVO = new RespVO();
            respVO.setSubMsg("分类下有记录，不能删除！如确认删除，先删除此分类下的数据");
            return respVO;
        } else {
            DataClassification classification = new DataClassification();
            classification.setId(id);
            classification.setState(0);
             this.updateById(classification);
             return null;
        }
    }

	@Override
	public List<DataClassification> getSubcategories(Long siteId, Long typeOne) {
		return dataClassificationMapper.getSubcategories(siteId,typeOne);
	}

	@Override
	public List<DataClassificationVo> getAllClassification(SiteComponyIdVO siteComponyIdVO, Long parentId) {
		Long siteId = siteComponyIdVO.getSiteId();
		log.info("查询的站点id:{}",siteId);
		Long createById = siteComponyIdVO.getCreateById();
		String createByName = siteComponyIdVO.getCreateByName();
		ClassificationVO classificationVO = null;
		Date date = new Date();
		int level = 0;
		if(parentId == 0){
			level = 0;
		} else {
			level = 1;
		}
		//获取根节点下的所有节点，只遍历二级
		List<DataClassificationVo> rootList = dataClassificationMapper.selectClassification(siteId, parentId);
		if (CollectionUtils.isNotEmpty(rootList)) {
			Long childParentId = null;
			//遍历根节点
			for(DataClassificationVo root : rootList){
				//获取子节点（一级）
				childParentId = root.getId();
				List<DataClassificationVo> childListOne = dataClassificationMapper.selectClassification(siteId, childParentId);
				if (CollectionUtils.isNotEmpty(childListOne)) {
					//遍历一级节点
					for(DataClassificationVo rootOne : childListOne){
						//获取子节点（二级）
						childParentId = rootOne.getId();
						List<DataClassificationVo> childListTwo = dataClassificationMapper.selectClassification(siteId, childParentId);
						if (CollectionUtils.isNotEmpty(childListTwo)) {
							rootOne.setChildClassifyNews(childListTwo);
							
							for(DataClassificationVo rootTwo : childListTwo){
								rootTwo.setLevel(level + 3);
							}
						}
						
						rootOne.setLevel(level + 2);
					}
					root.setChildClassifyNews(childListOne);
				}
				
				root.setLevel(level + 1);
			}
		}
		return rootList;
	}

}
