package com.yizhi.site.application.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.site.application.domain.DataClassification;
import com.yizhi.site.application.domain.EventTrack;
import com.yizhi.site.application.mapper.EventTrackMapper;
import com.yizhi.site.application.service.DataClassificationService;
import com.yizhi.site.application.service.EventTrackService;
import com.yizhi.site.application.vo.site.DayNumVo;
import com.yizhi.site.application.vo.site.EventTrackDayNumVo;
import com.yizhi.site.application.vo.site.EventTrackDeptNumVo;
import com.yizhi.site.application.vo.site.EventTrackTypeNumVo;
import com.yizhi.system.application.feign.DictionaryFeignClients;
import com.yizhi.system.application.vo.domain.Dictionary;
import com.yizhi.training.application.feign.LeaveWordClient;
import com.yizhi.training.application.vo.domain.AuthzUserGroupVo;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.util.application.domain.Response;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;

/**
 * com.yizhi.site.application.service.impl
 *
 * @author yanglang
 * @create 2025-01-14 10:15:57
 */
@Service
public class EventTrackServiceImpl extends ServiceImpl<EventTrackMapper, EventTrack> implements EventTrackService {
    private static final Logger logger= LoggerFactory.getLogger(EventTrackServiceImpl.class);

    @Autowired
    private DictionaryFeignClients dictionaryFeignClients;

    @Autowired
    private LeaveWordClient leaveWordClient;

    @Autowired
    private DataClassificationService dataClassificationService;

    @Autowired
    IdGenerator idGenerator;

    @Override
    public void addEvent(String type,Long businessId){
        logger.info("新增埋点{},{}",type,businessId);
        RequestContext context = ContextHolder.get();
        Long userId = context.getAccountId();
        if(Objects.equals(type,"event_login")){
            userId = businessId;
            ContextHolder.get().setAccountId(userId);
        }
        EventTrack eventTrack = new EventTrack();
        if(businessId!=null){
            DataClassification dataClassification = dataClassificationService.selectById(businessId);
            if(dataClassification!=null){
                eventTrack.setRemark1(dataClassification.getName());
            }
        }
        Date toDay = new Date();
        eventTrack.setBusinessId(businessId);
        eventTrack.setCreateById(userId);
        eventTrack.setCreateByName(context.getAccountName());
        eventTrack.setCreateTime(toDay);
        eventTrack.setUpdateById(userId);
        eventTrack.setUpdateByName(context.getAccountName());
        eventTrack.setUpdateTime(toDay);
        AuthzUserGroupVo authzUserGroupVo = leaveWordClient.getAuthzUserGroup(userId);
        if(authzUserGroupVo!=null){
            eventTrack.setDeptId(authzUserGroupVo.getId());
            eventTrack.setDeptName(authzUserGroupVo.getName());
        }
        String key = "event_track_type";
        List<Dictionary> dictionaryList = dictionaryFeignClients.listChildDictionary(key,false,1);
        if(CollUtil.isNotEmpty(dictionaryList)){
            Optional<Dictionary> optional = dictionaryList.stream().filter(d-> Objects.equals(d.getCode(),type)).findFirst();
            if(optional!=null&&optional.isPresent()){
                Dictionary dictionary = optional.get();
                eventTrack.setSubType(dictionary.getCnName());
                eventTrack.setType(Integer.valueOf(dictionary.getDescription()));
            }
        }
        eventTrack.setId(idGenerator.generate());
        this.insert(eventTrack);
    }

    @Override
    public EventTrackDayNumVo getTodayNum(String date) {
        if(ObjectUtils.isEmpty(date)){
            date = DateUtil.toDay(new Date());
        }
        return this.baseMapper.getTodayNum(date);
    }

    @Override
    public DayNumVo getNumByType(String date,List<Integer> types) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);
        return this.baseMapper.getNumByType(date,dateString,types);
    }

    @Override
    public List<EventTrackTypeNumVo> queryNumByType(String date,List<Integer> types) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);
        return this.baseMapper.queryNumByType(null,date,dateString,types);
    }

    @Override
    public List<EventTrackDeptNumVo> queryDeptByType(String date, List<Integer> types) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);

        List<EventTrackDeptNumVo> eventTrackDeptNumVoList = this.baseMapper.queryDeptByType(date,dateString,types);
        if(CollUtil.isNotEmpty(eventTrackDeptNumVoList)){
            String finalDate = date;
            eventTrackDeptNumVoList.stream().forEach(eventTrackDeptNumVo -> {
                eventTrackDeptNumVo.setEventTrackTypeNumVoList(this.baseMapper.queryNumByType(eventTrackDeptNumVo.getDeptId(),finalDate, dateString,types));
            });
        }
        return eventTrackDeptNumVoList;
    }

    public String getDateString(String date) {
        String dateString = DateUtil.toDay(new Date());
        String[] dateArrays = dateString.split("-");
        if(Objects.equals(date,"1")){
            dateString = dateArrays[0];
        }
        if(Objects.equals(date,"2")){
            dateString = dateArrays[0]+"-"+dateArrays[1];
        }
        if(Objects.equals(date,"4")){
            Calendar calendar = Calendar.getInstance();
            int week = calendar.get(Calendar.WEEK_OF_YEAR);
            if(week==1){
                dateString = String.valueOf(Integer.valueOf(dateArrays[0])-1);
            }else{
                dateString = dateArrays[0];
            }
        }
        return dateString;
    }
}
